@@ -1,6 +1,6 @@
 /*
  *  File-Roller
- * 
+ *
  *  Copyright (C) 2004 Free Software Foundation, Inc.
  *
  *  This library is free software; you can redistribute it and/or
@@ -18,7 +18,7 @@
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  *  Author: Paolo Bacchilega <paobac@cvs.gnome.org>
- * 
+ *
  */
 
 #include <config.h>
@@ -48,7 +48,7 @@ extract_to_callback (NautilusMenuItem *item,
 
 	uri = nautilus_file_info_get_uri (file);
 	default_dir = nautilus_file_info_get_parent_uri (file);
-	
+
 	cmd = g_string_new ("file-roller");
 	g_string_append_printf (cmd,
 				" --default-dir=%s --extract %s",
@@ -89,7 +89,7 @@ extract_here_callback (NautilusMenuItem *item,
 	for (scan = files; scan; scan = scan->next) {
 		NautilusFileInfo *file = scan->data;
 		char             *uri;
-		
+
 		uri = nautilus_file_info_get_uri (file);
 		g_string_append_printf (cmd, " %s", g_shell_quote (uri));
 		g_free (uri);
@@ -141,48 +141,118 @@ add_callback (NautilusMenuItem *item,
 
 
 static char *mime_types[] = {
+	"application/x-7z-compressed",
+ 	"application/x-7z-compressed-tar",
+ 	"application/x-ace",
+ 	"application/x-alz",
 	"application/x-ar",
 	"application/x-arj",
 	"application/x-bzip",
 	"application/x-bzip-compressed-tar",
+	"application/x-bzip1",
+	"application/x-bzip1-compressed-tar",
+	"application/x-cabinet",
+	"application/x-cbr",
+	"application/x-cbz",
+	"application/x-cd-image",
 	"application/x-compress",
 	"application/x-compressed-tar",
+	"application/x-cpio",
 	"application/x-deb",
+	"application/x-ear",
+	"application/x-ms-dos-executable",
 	"application/x-gtar",
 	"application/x-gzip",
+	"application/x-gzpostscript",
+	"application/x-java-archive",
 	"application/x-lha",
 	"application/x-lhz",
+	"application/x-lzma",
+	"application/x-lzma-compressed-tar",
+	"application/x-lzop",
+	"application/x-lzop-compressed-tar",
 	"application/x-rar",
 	"application/x-rar-compressed",
+	"application/x-rpm",
+	"application/x-rzip",
 	"application/x-tar",
+	"application/x-tarz",
+	"application/x-stuffit",
+	"application/x-war",
 	"application/x-zip",
 	"application/x-zip-compressed",
+	"application/x-zoo",
 	"application/zip",
 	"multipart/x-zip",
-	"application/x-rpm",
-	"application/x-jar",
-	"application/x-java-archive",
-	"application/x-lzma",
-	"application/x-lzop",
-	"application/x-zoo",
-	"application/x-cd-image",
-	"application/x-7z-compressed",
-	"application/x-gzpostscript",
-	"application/x-ms-dos-executable",
 	NULL
 };
 
 
-static gboolean
-is_archive (NautilusFileInfo *file)
+typedef struct
 {
-	int i;
+      gboolean is_archive;
+      gboolean is_derived_archive;
+} FileMimeInfo;
+
+
+static FileMimeInfo
+get_file_mime_info (NautilusFileInfo *file)
+{
+	FileMimeInfo file_mime_info;
+	int          i;
+		
+	file_mime_info.is_archive = FALSE;
+	file_mime_info.is_derived_archive = FALSE;
 
 	for (i = 0; mime_types[i] != NULL; i++)
-		if (nautilus_file_info_is_mime_type (file, mime_types[i]))
-			return TRUE;
+		if (nautilus_file_info_is_mime_type (file, mime_types[i])) {
+			char *mime_type;
+			char *content_type_mime_file;
+			char *content_type_mime_compare;
+		      
+			mime_type = nautilus_file_info_get_mime_type (file);
+		      
+			content_type_mime_file = g_content_type_from_mime_type (mime_type);
+			content_type_mime_compare = g_content_type_from_mime_type (mime_types[i]);
+		      
+			file_mime_info.is_archive = TRUE;
+			if ((content_type_mime_file != NULL) && (content_type_mime_compare != NULL))
+				file_mime_info.is_derived_archive = ! g_content_type_equals (content_type_mime_file, content_type_mime_compare);
+                  
+			g_free (mime_type);
+			g_free (content_type_mime_file);
+			g_free (content_type_mime_compare);
+		      
+			return file_mime_info;
+		}
 
-	return FALSE;
+	return file_mime_info;
+}
+
+
+static gboolean
+unsupported_scheme (NautilusFileInfo *file)
+{
+	gboolean  result = FALSE;
+	GFile    *location;
+	char     *scheme;
+
+	location = nautilus_file_info_get_location (file);
+	scheme = g_file_get_uri_scheme (location);
+
+	if (scheme != NULL) {
+		const char *unsupported[] = { "trash", "computer", NULL };
+		int         i;
+
+		for (i = 0; unsupported[i] != NULL; i++)
+			if (strcmp (scheme, unsupported[i]) == 0)
+				result = TRUE;
+	}
+
+	g_free (scheme);
+	g_object_unref (location);
+
+	return result;
 }
 
 
@@ -196,16 +266,27 @@ nautilus_fr_get_file_items (NautilusMenuProvider *provider,
 	gboolean  can_write = TRUE;
 	gboolean  one_item;
 	gboolean  one_archive = FALSE;
+	gboolean  one_derived_archive = FALSE;
 	gboolean  all_archives = TRUE;
+	gboolean  all_archives_derived = TRUE;
 
 	if (files == NULL)
 		return NULL;
 
-	for (scan = files; scan; scan = scan->next) {
-		NautilusFileInfo *file = scan->data;
+	if (unsupported_scheme ((NautilusFileInfo *) files->data))
+			return NULL;
 
-		if ((all_archives = is_archive (file)) == FALSE)
-			break;
+	for (scan = files; scan; scan = scan->next) {	
+		NautilusFileInfo *file = scan->data;
+		FileMimeInfo      file_mime_info;
+				
+		file_mime_info = get_file_mime_info (file);
+
+		if (all_archives && ! file_mime_info.is_archive)
+			all_archives = FALSE;
+	      
+		if (all_archives_derived && file_mime_info.is_archive && ! file_mime_info.is_derived_archive)
+			all_archives_derived = FALSE;
 
 		if (can_write) {
 			NautilusFileInfo *parent;
@@ -215,8 +296,11 @@ nautilus_fr_get_file_items (NautilusMenuProvider *provider,
 		}
 	}
 
+	/**/
+
 	one_item = (files != NULL) && (files->next == NULL);
 	one_archive = one_item && all_archives;
+	one_derived_archive = one_archive && all_archives_derived;
 
 	if (all_archives && can_write) {
 		NautilusMenuItem *item;
@@ -235,7 +319,7 @@ nautilus_fr_get_file_items (NautilusMenuProvider *provider,
 					(GDestroyNotify) nautilus_file_info_list_free);
 
 		items = g_list_append (items, item);
-	} 
+	}
 	else if (all_archives && ! can_write) {
 		NautilusMenuItem *item;
 
@@ -256,7 +340,7 @@ nautilus_fr_get_file_items (NautilusMenuProvider *provider,
 
 	}
 
-	if (! one_archive) {
+	if (! one_archive || one_derived_archive) {
 		NautilusMenuItem *item;
 
 		item = nautilus_menu_item_new ("NautilusFr::add",
