@@ -25,10 +25,7 @@
 #include <sys/types.h>
 #include <signal.h>
 #include <stdlib.h>
-
 #include <gio/gio.h>
-#include <glade/glade.h>
-
 #include "file-utils.h"
 #include "glib-utils.h"
 #include "fr-command.h"
@@ -306,7 +303,6 @@ main (int argc, char **argv)
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
 					   PKG_DATA_DIR G_DIR_SEPARATOR_S "icons");
 
-	glade_init ();
 	fr_stock_init ();
 	/* init_session (argv); */
 	startup_id = g_idle_add (startup_cb, NULL);
